@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/datetime.c,v 1.87.2.1 2002/02/25 16:22:48 thomas Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/utils/adt/datetime.c,v 1.87.2.3 2003/01/26 22:33:16 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -79,11 +79,11 @@ char	   *days[] = {"Sunday", "Monday", "Tuesday", "Wednesday",
 static datetkn datetktbl[] = {
 /*	text, token, lexval */
 	{EARLY, RESERV, DTK_EARLY}, /* "-infinity" reserved for "early time" */
+	{"abstime", IGNORE, 0},		/* for pre-v6.1 "Invalid Abstime" */
 	{"acsst", DTZ, 63},			/* Cent. Australia */
 	{"acst", DTZ, NEG(24)},		/* Atlantic/Porto Acre */
 	{"act", TZ, NEG(30)},		/* Atlantic/Porto Acre */
 	{DA_D, ADBC, AD},			/* "ad" for years >= 0 */
-	{"abstime", IGNORE, 0},		/* for pre-v6.1 "Invalid Abstime" */
 	{"adt", DTZ, NEG(18)},		/* Atlantic Daylight Time */
 	{"aesst", DTZ, 66},			/* E. Australia */
 	{"aest", TZ, 60},			/* Australia Eastern Std Time */
@@ -92,8 +92,8 @@ static datetkn datetktbl[] = {
 	{"akdt", DTZ, NEG(48)},		/* Alaska Daylight Time */
 	{"akst", DTZ, NEG(54)},		/* Alaska Standard Time */
 	{"allballs", RESERV, DTK_ZULU},		/* 00:00:00 */
-	{"almt", TZ, 36},			/* Almaty Time */
 	{"almst", TZ, 42},			/* Almaty Savings Time */
+	{"almt", TZ, 36},			/* Almaty Time */
 	{"am", AMPM, AM},
 	{"amst", DTZ, 30},			/* Armenia Summer Time (Yerevan) */
 	{"amt", TZ, 24},			/* Armenia Time (Yerevan) */
@@ -102,6 +102,8 @@ static datetkn datetktbl[] = {
 #endif
 	{"anast", DTZ, 78},			/* Anadyr Summer Time (Russia) */
 	{"anat", TZ, 72},			/* Anadyr Time (Russia) */
+	{"apr", MONTH, 4},
+	{"april", MONTH, 4},
 #if 0
 aqtst
 aqtt
@@ -112,8 +114,6 @@ arst
 ashst
 ast /* Atlantic Standard Time, Arabia Standard Time, Acre Standard Time */
 #endif
-	{"apr", MONTH, 4},
-	{"april", MONTH, 4},
 	{"ast", TZ, NEG(24)},		/* Atlantic Std Time (Canada) */
 	{"at", IGNORE, 0},			/* "at" (throwaway) */
 	{"aug", MONTH, 8},
@@ -171,12 +171,12 @@ cost
 #endif
 	{"cot", TZ, NEG(30)},		/* Columbia Time */
 	{"cst", TZ, NEG(36)},		/* Central Standard Time */
+	{DCURRENT, RESERV, DTK_CURRENT},	/* "current" is always now */
 #if 0
 cvst
 #endif
 	{"cvt", TZ, 42},			/* Christmas Island Time (Indian Ocean) */
 	{"cxt", TZ, 42},			/* Christmas Island Time (Indian Ocean) */
-	{DCURRENT, RESERV, DTK_CURRENT},	/* "current" is always now */
 	{"d", UNITS, DTK_DAY},		/* "day of month" for ISO input */
 	{"davt", TZ, 42},			/* Davis Time (Antarctica) */
 	{"ddut", TZ, 60},			/* Dumont-d'Urville Time (Antarctica) */
@@ -402,8 +402,8 @@ sgt
 syot
 #endif
 	{"t", ISOTIME, DTK_TIME},	/* Filler for ISO time fields */
-	{"that", TZ, NEG(60)},		/* Tahiti Time */
 	{"tft", TZ, 30},			/* Kerguelen Time */
+	{"that", TZ, NEG(60)},		/* Tahiti Time */
 	{"thu", DOW, 4},
 	{"thur", DOW, 4},
 	{"thurs", DOW, 4},
@@ -504,9 +504,9 @@ static datetkn deltatktbl[] = {
 	{DDAY, UNITS, DTK_DAY},		/* "day" relative */
 	{"days", UNITS, DTK_DAY},	/* "days" relative */
 	{"dec", UNITS, DTK_DECADE}, /* "decade" relative */
-	{"decs", UNITS, DTK_DECADE},	/* "decades" relative */
 	{DDECADE, UNITS, DTK_DECADE},		/* "decade" relative */
 	{"decades", UNITS, DTK_DECADE},		/* "decades" relative */
+	{"decs", UNITS, DTK_DECADE},	/* "decades" relative */
 	{"h", UNITS, DTK_HOUR},		/* "hour" relative */
 	{DHOUR, UNITS, DTK_HOUR},	/* "hour" relative */
 	{"hours", UNITS, DTK_HOUR}, /* "hours" relative */
@@ -522,7 +522,6 @@ static datetkn deltatktbl[] = {
 	{"mils", UNITS, DTK_MILLENNIUM},	/* "millennia" relative */
 	{"min", UNITS, DTK_MINUTE}, /* "minute" relative */
 	{"mins", UNITS, DTK_MINUTE},	/* "minutes" relative */
-	{"mins", UNITS, DTK_MINUTE},	/* "minutes" relative */
 	{DMINUTE, UNITS, DTK_MINUTE},		/* "minute" relative */
 	{"minutes", UNITS, DTK_MINUTE},		/* "minutes" relative */
 	{"mon", UNITS, DTK_MONTH},	/* "months" relative */
@@ -543,7 +542,6 @@ static datetkn deltatktbl[] = {
 	{"seconds", UNITS, DTK_SECOND},
 	{"secs", UNITS, DTK_SECOND},
 	{DTIMEZONE, UNITS, DTK_TZ}, /* "timezone" time offset */
-	{"timezone", UNITS, DTK_TZ},	/* "timezone" time offset */
 	{"timezone_h", UNITS, DTK_TZ_HOUR}, /* timezone hour units */
 	{"timezone_m", UNITS, DTK_TZ_MINUTE},		/* timezone minutes units */
 	{"undefined", RESERV, DTK_INVALID}, /* pre-v6.1 invalid time */
@@ -1427,13 +1425,15 @@ DecodeDateTime(char **field, int *ftype, int nf,
 
 
 /* DetermineLocalTimeZone()
+ *
  * Given a struct tm in which tm_year, tm_mon, tm_mday, tm_hour, tm_min, and
  * tm_sec fields are set, attempt to determine the applicable local zone
  * (ie, regular or daylight-savings time) at that time.  Set the struct tm's
  * tm_isdst field accordingly, and return the actual timezone offset.
  *
- * This subroutine exists mainly to centralize uses of mktime() and defend
- * against mktime() bugs on various platforms...
+ * This subroutine exists to centralize uses of mktime() and defend against
+ * mktime() bugs/restrictions on various platforms.  This should be
+ * the *only* call of mktime() in the backend.
  */
 int
 DetermineLocalTimeZone(struct tm * tm)
@@ -1441,7 +1441,10 @@ DetermineLocalTimeZone(struct tm * tm)
 	int			tz;
 
 	if (HasCTZSet)
+	{
+		tm->tm_isdst = 0;		/* for lack of a better idea */
 		tz = CTimeZone;
+	}
 	else if (IS_VALID_UTIME(tm->tm_year, tm->tm_mon, tm->tm_mday))
 	{
 #if defined(HAVE_TM_ZONE) || defined(HAVE_INT_TIMEZONE)
@@ -1463,20 +1466,90 @@ DetermineLocalTimeZone(struct tm * tm)
 		/* indicate timezone unknown */
 		tmp->tm_isdst = -1;
 
-		mktime(tmp);
-
-		tm->tm_isdst = tmp->tm_isdst;
+		if (mktime(tmp) != ((time_t) -1) &&
+			tmp->tm_isdst >= 0)
+		{
+			/* mktime() succeeded, trust its result */
+			tm->tm_isdst = tmp->tm_isdst;
 
 #if defined(HAVE_TM_ZONE)
-		/* tm_gmtoff is Sun/DEC-ism */
-		if (tmp->tm_isdst >= 0)
+			/* tm_gmtoff is Sun/DEC-ism */
 			tz = -(tmp->tm_gmtoff);
-		else
-			tz = 0;				/* assume UTC if mktime failed */
 #elif defined(HAVE_INT_TIMEZONE)
-		tz = ((tmp->tm_isdst > 0) ? (TIMEZONE_GLOBAL - 3600) : TIMEZONE_GLOBAL);
+			tz = ((tmp->tm_isdst > 0) ? (TIMEZONE_GLOBAL - 3600) : TIMEZONE_GLOBAL);
 #endif   /* HAVE_INT_TIMEZONE */
+		}
+		else
+		{
+			/*
+			 * We have a buggy (not to say deliberately brain damaged)
+			 * mktime().  Work around it by using localtime() instead.
+			 *
+			 * First, generate the time_t value corresponding to the given
+			 * y/m/d/h/m/s taken as GMT time.  This will not overflow (at
+			 * least not for time_t taken as signed) because of the range
+			 * check we did above.
+			 */
+			long		day,
+						mysec,
+						locsec,
+						delta1,
+						delta2;
+			time_t		mytime;
 
+			day = (date2j(tm->tm_year, tm->tm_mon, tm->tm_mday) -
+				   date2j(1970, 1, 1));
+			mysec = tm->tm_sec + (tm->tm_min + (day * 24 + tm->tm_hour) * 60) * 60;
+			mytime = (time_t) mysec;
+			/*
+			 * Use localtime to convert that time_t to broken-down time, and
+			 * reassemble to get a representation of local time.
+			 */
+			tmp = localtime(&mytime);
+			day = (date2j(tmp->tm_year + 1900, tmp->tm_mon + 1, tmp->tm_mday) -
+				   date2j(1970, 1, 1));
+			locsec = tmp->tm_sec + (tmp->tm_min + (day * 24 + tmp->tm_hour) * 60) * 60;
+			/*
+			 * The local time offset corresponding to that GMT time is
+			 * now computable as mysec - locsec.
+			 */
+			delta1 = mysec - locsec;
+			/*
+			 * However, if that GMT time and the local time we are actually
+			 * interested in are on opposite sides of a daylight-savings-time
+			 * transition, then this is not the time offset we want.  So,
+			 * adjust the time_t to be what we think the GMT time corresponding
+			 * to our target local time is, and repeat the localtime() call
+			 * and delta calculation.  We may have to do it twice before we
+			 * have a trustworthy delta.
+			 *
+			 * Note: think not to put a loop here, since if we've been given
+			 * an "impossible" local time (in the gap during a spring-forward
+			 * transition) we'd never get out of the loop.  Twice is enough
+			 * to give the behavior we want, which is that "impossible" times
+			 * are taken as standard time, while at a fall-back boundary
+			 * ambiguous times are also taken as standard.
+			 */
+			mysec += delta1;
+			mytime = (time_t) mysec;
+			tmp = localtime(&mytime);
+			day = (date2j(tmp->tm_year + 1900, tmp->tm_mon + 1, tmp->tm_mday) -
+				   date2j(1970, 1, 1));
+			locsec = tmp->tm_sec + (tmp->tm_min + (day * 24 + tmp->tm_hour) * 60) * 60;
+			delta2 = mysec - locsec;
+			if (delta2 != delta1)
+			{
+				mysec += (delta2 - delta1);
+				mytime = (time_t) mysec;
+				tmp = localtime(&mytime);
+				day = (date2j(tmp->tm_year + 1900, tmp->tm_mon + 1, tmp->tm_mday) -
+					   date2j(1970, 1, 1));
+				locsec = tmp->tm_sec + (tmp->tm_min + (day * 24 + tmp->tm_hour) * 60) * 60;
+				delta2 = mysec - locsec;
+			}
+			tm->tm_isdst = tmp->tm_isdst;
+			tz = (int) delta2;
+		}
 #else							/* not (HAVE_TM_ZONE || HAVE_INT_TIMEZONE) */
 		tm->tm_isdst = 0;
 		tz = CTimeZone;
