@@ -3,7 +3,7 @@
  *
  * Copyright 2000 by PostgreSQL Global Development Group
  *
- * $Header: /home/projects/pgsql/cvsroot/pgsql/src/bin/psql/describe.h,v 1.9 2000/04/12 17:16:22 momjian Exp $
+ * $Header: /cvsroot/pgsql/src/bin/psql/describe.h,v 1.13 2001/11/05 17:46:31 momjian Exp $
  */
 #ifndef DESCRIBE_H
 #define DESCRIBE_H
@@ -22,6 +22,9 @@ bool		describeTypes(const char *name, bool verbose);
 /* \do */
 bool		describeOperators(const char *name);
 
+/* \du */
+bool		describeUsers(const char *name);
+
 /* \z (or \dp) */
 bool		permissionsList(const char *name);
 
@@ -37,4 +40,4 @@ bool		listAllDbs(bool desc);
 /* \dt, \di, \ds, \dS, etc. */
 bool		listTables(const char *infotype, const char *name, bool desc);
 
-#endif	 /* DESCRIBE_H */
+#endif   /* DESCRIBE_H */
